const Splitter = function(store) {
    this.store = store;
    this.writer = new XMLSerializer();
    this.todo = [];
}

Splitter.prototype.split = function(fileName, elt) {
    var errs = [];
    this.recsplit(fileName, elt, null, "", errs);
    while (this.todo.length > 0) {
        const hd = this.todo.pop();
        this.recsplit(fileName, hd.extract, hd.md, "", errs);
    }
    return JSON.stringify(errs);
}

// TODO: we should capture errors generally
// TODO: we should check for proper nesting, e.g.
//   containers may not contain punnets or cards
//   names aren't duplicated
Splitter.prototype.recsplit = function(fileName, elt, md, fileLabel, errors) {
    for (var i=0;i<elt.children.length;i++) {
        var lmd = md;
        var lerr = errors;
        var c = elt.children[i];
        if (c.id) {
        	var etext = this.captureText(c);
            const id = c.id;
            c.removeAttribute("id");
            var saveAs = null;
            if (id.startsWith("flas-card-")) {
                saveAs = id.replace("flas-card-", "cards/");
                lmd = {};
                lerr = [];
            } else if (id.startsWith("flas-item-")) {
                saveAs = id.replace("flas-item-", "items/");
                lmd = {};
                lerr = [];
            } else if (id.startsWith("flas-content-")) {
                const pid = id.replace("flas-content-", "");
                if (md == null) {
                    errors.push("field declaration not in card or item: " + pid);
                    errors.push(etext);
                    continue;
                }
                if (md[pid]) {
                    errors.push("duplicate field in '" + fileLabel + "': " + pid);
                    errors.push(etext);
                }
                if (c.tagName == "IMG") {
                    // wrap it in in span for targeting purposes
                    var ps = document.createElement("span");
                    ps.className = "flas-image-parent";
                    c.parentElement.insertBefore(ps, c);
                    ps.appendChild(c);
                    md[pid] = 'image';
                } else
                    md[pid] = 'content';
                c.setAttribute("data-flas-" + md[pid], pid);
                var extract = document.createDocumentFragment();
                while (c.firstChild) {
                    const tmp = c.removeChild(c.firstChild);
                    extract.appendChild(tmp);
                }
                if (extract.children.length > 0) {
                    this.todo.push({ extract, md });
                }
            } else if (id.startsWith("flas-style-")) {
                const pid = id.replace("flas-style-", "");
                if (md == null) {
                    errors.push("field declaration not in card or item: " + pid);
                    errors.push(etext);
                    continue;
                }
                if (md[pid]) {
                    errors.push("duplicate field in '" + fileLabel + "': " + pid);
                    errors.push(etext);
                }
                c.setAttribute("data-flas-style", pid);
                md[pid] = 'style';
            } else if (id.startsWith("flas-punnet-")) {
                const pid = id.replace("flas-punnet-", "");
                if (md == null) {
                    errors.push("field declaration not in card or item: " + pid);
                    errors.push(etext);
                    continue;
                }
                if (md[pid]) {
                    errors.push("duplicate field in '" + fileLabel + "': " + pid);
                    errors.push(etext);
                }
                c.setAttribute("data-flas-punnet", pid);
                var extract = document.createDocumentFragment();
                while (c.firstChild) {
                    const tmp = c.removeChild(c.firstChild);
                    extract.appendChild(tmp);
                }
                if (extract.children.length > 0) {
                    this.todo.push({ extract, md });
                }
                md[pid] = 'punnet';
            } else if (id.startsWith("flas-container-")) {
                const pid = id.replace("flas-container-", "");
                if (md == null) {
                    errors.push("field declaration not in card or item: " + pid);
                    errors.push(etext);
                    continue;
                }
                if (md[pid]) {
                    errors.push("duplicate field in '" + fileLabel + "': " + pid);
                    errors.push(etext);
                }
                c.setAttribute("data-flas-container", pid);
                var extract = document.createDocumentFragment();
                while (c.firstChild) {
                    const tmp = c.removeChild(c.firstChild);
                    extract.appendChild(tmp);
                }
                if (extract.children.length > 0) {
                    this.todo.push({ extract, md });
                }
                md[pid] = 'container';
            }
            this.recsplit(fileName, c, lmd, saveAs, lerr);
            if (saveAs) {
                this.store.save(fileName, saveAs + ".html", this.writer.serializeToString(c).replace(' xmlns="http://www.w3.org/1999/xhtml"', ''), "");
                this.store.save(fileName, saveAs + ".json", JSON.stringify(lmd), JSON.stringify(lerr));
            }
        } else {
            this.recsplit(fileName, c, lmd, saveAs, lerr);
        }
    }
}

Splitter.prototype.captureText = function(elt) {
	var cln = elt.cloneNode(true);
	while (cln.firstChild)
		cln.removeChild(cln.firstChild);
	return this.writer.serializeToString(cln).replace(' xmlns="http://www.w3.org/1999/xhtml"', '').replace(/<\/.*/, "");
}
if (window)
    window.Splitter = Splitter;